const path = require('path');

module.exports = (api, opts = {}) => {
  const env = process.env.NODE_ENV || 'development';
  const isDev = env === 'development';
  const isProd = env === 'production';
  const isTest = env === 'test';
  const useTs = typeof opts.ts === 'boolean' ? opts.ts : false
  const isVue = typeof opts.vue === 'boolean' ? opts.vue : true
  const isReact = !isVue  

  const absoluteRuntimepath = path.dirname(require.resolve('@babel/runtime/package.json'));

  const presets = [
    isTest && [
      require('@babel/preset-env').default,
      {
        targets: {
          node: 'current'
        }
      }
    ],
    (isProd || isDev) && [
      require('@babel/preset-env').default,
      {
        useBuiltIns: 'entry',
        corejs: 3,
        modules: false,
        targets: {
          browsers: [
            '> 1%',
            'last 2 versions',
            'Firefox ESR',
          ]
        },
        exclude: ['transform-typeof-symbol']
      }
    ],
    isVue && [require('@vue/babel-preset-jsx')],
    isReact && [
      require('@babel/preset-react').default,
      {
        development: isDev || isTest,
        useBuiltIns: true,
      },
    ],
    useTs && [require('@babel/preset-typescript').default],
  ].filter(Boolean);

  const plugins = [
    require('@babel/plugin-syntax-jsx').default,
    require('babel-plugin-macros'),
    [
      require('@babel/plugin-transform-destructuring').default,
      {
        loose: false
      }
    ],
    [
      require('@babel/plugin-proposal-class-properties').default,
      {
        loose: true,
      },
    ],
    [
      require('@babel/plugin-proposal-object-rest-spread').default,
      {
        useBuiltIns: true,
      },
    ],
    [
      require('@babel/plugin-proposal-decorators').default,
      {
        legacy: true
      },
    ],
    isDev && [
      require('@babel/plugin-transform-runtime').default,
      {
        corejs: false,
        helpers: true,
        regenerator: true,
        useESModules: isDev,
        absoluteRuntime: absoluteRuntimepath,
      },
    ],
    isProd && [
      require('@babel/plugin-transform-regenerator').default
    ],
    require('@babel/plugin-syntax-dynamic-import').default,
    isTest && require('babel-plugin-dynamic-import-node')
  ].filter(Boolean);

  return {
    presets,
    plugins
  };
};